package cz.cvut.felk.newsgroup.preprocess;

import java.io.*;
import java.util.*;

public class Main {

    /**
     * Default path to the newsgroup training dataset
     */
    static String trainPath = "20news-bydate-train";

    /**
     * Default path to the newsgroup training dataset
     */
    static String testPath = "20news-bydate-test";

    /**
     * Default path to the newsgroup training dataset
     */
    static String trainArff = "20news-bydate-train.arff";

    /**
     * Default path to the newsgroup training dataset
     */
    static String testArff = "20news-bydate-test.arff";

    /**
     * Runs the whole programme
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            initFromCommandLine(args);
        } catch(IllegalArgumentException exc) {
            System.err.println(exc.getMessage());
            return;
        }

        Map<File,String> tmp;
        try {
            tmp = readInputDataset(new File(trainPath));
        } catch(IOException e) {
            System.err.println("Directory with training data does not exist.");
            return;
        }
        final Map<File,String> trainData = tmp;

        try {
            tmp = readInputDataset(new File(testPath));
        } catch(IOException e) {
            System.err.println("Directory with testing data does not exist.");
          return;
        }
        final Map<File,String> testData = tmp;

        ///////////////////////////////////////////////////////////
        // PHASE 1: FINDING THE APPROPRIATE WORDS INTO THE MODEL //
        ///////////////////////////////////////////////////////////
        
        final ModelBuilder builder = new ModelBuilder();

        try {
            
            for (File document : trainData.keySet()) {
                String newsgroup = trainData.get(document);
                
                // Put each training example in the model.
                BufferedReader documentContent = new BufferedReader(
                                                 new FileReader(document));
                builder.parseFile(newsgroup, documentContent);
            }
            
        } catch(IOException exc) {
            System.err.println("Model cannot be constructed"
                    + ", error while reading an training example: "
                    + exc.getMessage());
            return;
        }

        //////////////////////////////////
        // PHASE 2: CONSTRUCT THE MODEL //
        //////////////////////////////////

        final Model model = builder.createModel();

        ///////////////////////////////////
        // PHASE 3: PRINT THE ARFF FILES //
        ///////////////////////////////////

        Thread t1 = new Thread() { @Override public void run() {
            writeOutputDataset(trainData, model, trainArff);
        }};
        Thread t2 = new Thread() { @Override public void run() {
            writeOutputDataset(testData, model, testArff);
        }};

        try {
            t1.start();
            t2.start();
            t1.join(0);
            t2.join(0);
        } catch (InterruptedException e) {
            System.err.println("Interrupted while writing output files: "
                                                        +e.getMessage());
        }
    }

    /**
     * Writes the while dataset into an ARFF file
     *
     * @param data
     * @param model
     * @param outputFileName
     */
    private static void writeOutputDataset(Map<File,String> data,
                                        Model model, String outputFileName) {
        try {
            FileWriter fileWriter = new FileWriter(outputFileName);
            BufferedWriter writer = new BufferedWriter(fileWriter);
            writer.write(getArffHeader(model));

            for (File document : data.keySet()) {
                String newsgroup = data.get(document);

                List<Boolean> instance = null;
                try {
                    BufferedReader documentContent = new BufferedReader(
                                                     new FileReader(document));

                    instance = model.fileToVector(documentContent);
                } catch(IOException exc) {
                    System.err.println("Cannot read from file: '"
                            + exc.getMessage() + "'");
                    continue;
                }


                for (Boolean value : instance) {
                    writer.append('\'');
                    writer.append(value ? "true" : "false");
                    writer.append("',");
                }
                writer.append("'"+newsgroup+"'");
                writer.newLine();
            }

            writer.close();
            
        } catch(IOException exc) {
            System.err.println("Cannot write to file: '"
                    + outputFileName + "'.");
            return;
        }
    }

    /**
     * Finds all examples in a dataset + their target class
     *
     * @param root the path to the dataset
     * @return the list all files (=examples) + appropriate target class
     * @throws IOException the root does not exist
     */
    private static Map<File,String> readInputDataset(File root)
                                                        throws IOException {
        Map<File,String> output = new HashMap<File, String>();
        File[] newsgroups;
        try {
            newsgroups = root.listFiles();
            for (File newsgroup : newsgroups)
                for (File document : newsgroup.listFiles())
                    output.put(document, newsgroup.getName());
        } catch (NullPointerException exc) {
            throw new IOException("Directory does not exist.", exc);
        }


        return output;
    }

    /**
     * Parse command-line parameters.
     *
     * @param args array of command-line parameters
     */
    private static void initFromCommandLine(String[] args) {
        int i = 0;
        while (i < args.length) {
            String param;

            param = "--train-dir=";
            if (args[i].startsWith(param)) {
                trainPath = args[i].substring(param.length());
                i++; continue;
            }

            param = "--test-dir=";
            if (args[i].startsWith(param)) {
                testPath = args[i].substring(param.length());
                i++; continue;
            }

            param = "--train-output=";
            if (args[i].startsWith(param)) {
                trainArff = args[i].substring(param.length());
                i++; continue;
            }

            param = "--test-output=";
            if (args[i].startsWith(param)) {
                testArff = args[i].substring(param.length());
                i++; continue;
            }
            throw new IllegalArgumentException("Unknown argument: '"+args[i]+"'.");
        }
    }

    /**
     * Produces a header for an ARFF file.
     *
     * @param model the model containing column definitions
     * @return string to be written into the file
     */
    private static String getArffHeader(Model model) {
        StringBuilder output = new StringBuilder();

        output.append("% Model dimensionality: ").append(model.size()).append('\n');

        output.append("@relation 'newsgroup20'\n");

        for (String word : model)
            output.append("@attribute '" + word + "' {'true','false'}\n");

        // Build a set of classes
        Set<String> classes = new HashSet<String>();
        for (String rootPath : new String[] {testPath, trainPath })
            for (File newsgroup : (new File(rootPath)).listFiles()) {
                if (! newsgroup.isDirectory()) continue;
                classes.add(newsgroup.getName());
            }

        output.append("@attribute 'targetClass' {");
        boolean first = true;
        for (String newsgroup : classes) {
            if (! first) output.append(",");
            output.append('\'').append(newsgroup).append('\'');
            first = false;
        }
        output.append("}\n");

        output.append("@data");
        output.append("\n");

        return output.toString();
    }
}
