package cz.cvut.felk.newsgroup.preprocess;

import java.io.*;
import java.util.*;
import static cz.cvut.felk.newsgroup.preprocess.StringUtils.*;

/**
 * Helper class used to build a model from thes set of training examples.
 *
 * <p>This class consumes files from the testing set and produces a model
 * at its output.</p>
 */
public class ModelBuilder {

    private static final int LIMIT = 200;

    /**
     * Partially constructed model.
     *
     * <p>Currently the partial model is a set of words. In order to
     * improve it, you can start "counting the frequency" of each word.</p>
     */
    private Set<String> partialModel = new HashSet<String>(LIMIT);

    /**
     * Parses the given file and extracts the information into the model.
     *
     * <p>Currently the method only takes the set of words in a file regardless
     * of the newsgroup, from which the file comes from. One possible idea
     * is to focus on such words, which have "interesting" statistical
     * distribution among different newsgroups.</p>
     *
     * <p>Another suggestion: You can to use the natural language parser
     * and extract the subject and verb from the each sentence.
     * See <tt>[Project Home]/lib/stanford-parser/ParserDemo.java</tt>
     * for inspiration.</p>
     *
     * @param targetClass
     * @param fileContent
     * @throws IOException
     */
    public void parseFile(String targetClass, BufferedReader fileContent)
                                                    throws IOException{

        /* We will only accept first 100 words. Without this limit,
         * the vector would have around 12000 words, which is too much.
         *
         * TODO: Implement a better criterion than "first X words".
         */
        if (partialModel.size() >= LIMIT) return;

        for (String word : tokenize( fileContent.readLine() )) {
            partialModel.add( processWord(word) );
            if (partialModel.size() >= LIMIT) return;
        }
    }

    public Model createModel() {
        Model model = new Model();

        partialModel.remove(null);
        partialModel.remove("");
        model.addAll(partialModel);
        Collections.sort(model);
        
        return model;
    }

}
