package cz.cvut.felk.newsgroup.preprocess;

/**
 * Different string-processing utilities.
 */
public abstract class StringUtils {

    /**
     * Tokenizer splits a line into words.
     * 
     * @param lineWithWords a line in a document
     * @return words on the given line
     */
    public static String[] tokenize(String lineWithWords) {
        return lineWithWords.split("[ !\\.?]");
    }

    /**
     * Converts a raw word read from a file into a normalized form.
     *
     * @param word the word read from a file.
     * @return normalized form of the given word.
     */
    public static String processWord(String word) {
        word = word.toLowerCase();
        word = simplify(word);
        word = stem(word);
        return word;
    }

    /**
     * Returns the "stem" of a word.
     *
     * <p>A stemmer for English should identify the string "cats"
     * (and possibly "catlike", "catty" etc.) as based on the root "cat",
     * and "stemmer", "stemming", "stemmed" as based on "stem".
     * A stemming algorithm reduces the words "fishing", "fished", "fish",
     * and "fisher" to the root word, "fish".</p>
     *
     * <p>You can read and consult
     * <a href='http://en.wikipedia.org/wiki/Stemming#Suffix_Stripping_Algorithms'
     * >article on wikipedia</a>.</p>
     *
     * @param word an English word in lower-case
     * @return the stem of the given word
     */
    private static String stem(String word) {
        final String[] suffixes = new String[] {"ed", "s", "ing"};

        for (String suffix : suffixes)
            if (word.endsWith(suffix))
                return word.substring(0, word.length()-suffix.length());

        return word;
    }

    /**
     * Removes all characters, which are not letters.
     *
     * @param word a lower-case string
     * @return the word stripped of non-letter charactes
     */
    private static String simplify(String word) {
        StringBuilder output = new StringBuilder();

        for (char ch : word.toLowerCase().toCharArray())
            if (ch >= 'a' && ch <='z')
                output.append(ch);

        return output.toString();
    }

}
